@@ -127,6 +127,35 @@
         }
     };
 
+
+    var calcMinSize = function calcMinSize(size) {
+        var minSize = [size.min_w || size.w || 0, size.min_h || size.h || 0];
+        $.each(size.auto || [], function(i, autoSize) {
+            if (!autoSize.required) {
+                return;
+            }
+            var min_w = autoSize.min_w || autoSize.w || 0;
+            var min_h = autoSize.min_h || autoSize.h || 0;
+            minSize[0] = Math.max(minSize[0], min_w);
+            minSize[1] = Math.max(minSize[1], min_h);
+        });
+        return minSize;
+    };
+    // Fill in 'Min Size' help text
+    $(document).ready(function(){
+        var isStandalone = $('body').is('.cropduster-standalone');
+        var sizes = JSON.parse($('#id_crop-sizes').val());
+        if (isStandalone || !sizes) {
+            return;
+        }
+        var minSizes = sizes.map(calcMinSize);
+        var largest = [
+            Math.max.apply(null, minSizes.map(function(s) { return s[0]; })) || 1,
+            Math.max.apply(null, minSizes.map(function(s) { return s[1]; })) || 1
+        ];
+        $('#upload-min-size-help').html('Min. size: ' + largest.join(' x '));
+    });
+
     var CropBoxClass = Class.extend({
         jcrop: undefined,
         data: {},
@@ -149,7 +178,7 @@
                     w = options.minSize[0];
                 }
                 if (Math.abs(h - options.minSize[1]) == 1) {
-                    w = options.minSize[1];
+                    h = options.minSize[1];
                 }
             }
             if (options.maxSize) {
@@ -157,7 +186,7 @@
                     w = options.maxSize[0];
                 }
                 if (Math.abs(h - options.maxSize[1]) == 1) {
-                    w = options.maxSize[1];
+                    h = options.maxSize[1];
                 }
             }
             $('#id_thumbs-' + i + '-crop_x').val(x);
@@ -226,30 +255,71 @@
                 this.setCropOptions(sizeData);
             }
         },
+        getAspectRatioExtent: function(size) {
+            var aspect = (!size.is_auto && size.w && size.h) ? (size.w / size.h) : 0,
+                minAspect = aspect,
+                maxAspect = aspect || Infinity;
+
+            if (size.w && size.min_h > 1) {
+                maxAspect = Math.min(maxAspect, size.w / size.min_h);
+            }
+            if (size.w && size.max_h) {
+                minAspect = Math.max(minAspect, size.w / size.max_h);
+            }
+            if (size.h && size.min_w > 1) {
+                minAspect = Math.max(minAspect, size.min_w / size.h);
+            }
+            if (size.h && size.max_w) {
+                maxAspect = Math.min(maxAspect, size.max_w / size.h);
+            }
+
+            return {
+                min: minAspect,
+                max: maxAspect
+            };
+        },
         setCropOptions: function(size) {
             if (!size || typeof size != 'object') {
                 return;
             }
-            var aspectRatio = (size.w && size.h) ? (size.w / size.h) : 0;
-            var minSize = [size.min_w || size.w || 0, size.min_h || size.h || 0];
+            var self = this;
+
+            var aspectRatio = (size.w && size.h) ? (size.w / size.h) : 0,
+                minAspectRatio = aspectRatio,
+                maxAspectRatio = aspectRatio || Infinity;
+            var minSize = {
+                w: size.min_w || size.w || 0,
+                h: size.min_h || size.h || 0
+            };
+
+            var aspectExtent = this.getAspectRatioExtent(size);
+
             $.each(size.auto || [], function(i, autoSize) {
-                var min_w = autoSize.min_w || autoSize.w || 0;
-                var min_h = autoSize.min_h || autoSize.h || 0;
-                minSize[0] = Math.max(minSize[0], min_w);
-                minSize[1] = Math.max(minSize[1], min_h);
+                minSize.w = Math.max(minSize.w, autoSize.min_w || autoSize.w || 0);
+                minSize.h = Math.max(minSize.h, autoSize.min_h || autoSize.h || 0);
             });
+
+            if (aspectExtent.max == Infinity) {
+                aspectExtent.max = 0;
+            }
+
             var options = {
-                aspectRatio: aspectRatio,
                 boxWidth: $('#cropbox').width(),
                 boxHeight: $('#cropbox').height(),
-                minSize: minSize,
+                minSize: calcMinSize(size),
                 trueSize: [this.orig_w, this.orig_h],
-                setSelect: this.getCropSelect(aspectRatio)
+                setSelect: this.getCropSelect(aspectRatio, aspectExtent),
+                bgColor: '#ffffff'
             };
+            if (aspectRatio) {
+                options.aspectRatio = aspectRatio;
+            } else {
+                options.minAspectRatio = aspectExtent.min;
+                options.maxAspectRatio = aspectExtent.max;
+            }
             if (this.jcrop) {
                 this.jcrop.setOptions(options);
             } else {
-                var self = this;
                 options = $.extend(options, {
                     onSelect: function(c) { self.updateCoordinates(c, self.index); }
                 });
@@ -265,7 +335,7 @@
             $('#upload-footer').hide();
             $('#crop-footer').show();
         },
-        getCropSelect: function(aspectRatio) {
+        getCropSelect: function(aspectRatio, aspectExtent) {
             var x, y, w, h;
             var thumbData = ($.isArray(this.data.thumbs)) ? this.data.thumbs[this.index] : {};
             if (typeof thumbData == 'object' && thumbData.crop_w && thumbData.crop_h) {
@@ -280,7 +350,14 @@
                 y = 0;
                 w = this.orig_w;
                 h = this.orig_h;
-            } else if ((this.orig_w / this.orig_h) < aspectRatio) {
+                aspectRatio = w / h;
+                if (aspectExtent.min && aspectRatio < aspectExtent.min) {
+                    aspectRatio = aspectExtent.min;
+                } else if (aspectExtent.max && aspectRatio > aspectExtent.max) {
+                    aspectRatio = aspectExtent.max;
+                }
+            }
+            if ((this.orig_w / this.orig_h) < aspectRatio) {
                 // The uploaded image is taller than the needed aspect ratio
                 x = 0;
                 w = this.orig_w;
@@ -351,24 +428,31 @@
         if (Object.prototype.toString.call(data.thumbs) != '[object Array]') {
             return;
         }
+
+        var initialFormCount = 0;
+
         for (var i = 0; i < data.thumbs.length; i++) {
             for (field in data.thumbs[i]) {
                 var value = data.thumbs[i][field];
+                if (field == 'id' && value) {
+                    initialFormCount++;
+                }
                 if (Object.prototype.toString.call(value).match(/\[object (Object|Array)\]/)) {
                     value = JSON.stringify(value);
                 }
                 var $input = $('#id_thumbs-' + i + '-' + field);
                 if ($input.attr('type') == 'checkbox') {
                     if (value && value != 'off' && value != 'false' && value != '0') {
-                        $input.prop('checked', true);
+                        $input[0].checked = true;
                     } else {
-                        $input.removeProp('checked');
+                        delete $input[0].checked;
                     }
                 } else {
                     $input.val(value);
                 }
             }
         }
+        $('#id_thumbs-INITIAL_FORMS').val(initialFormCount);
     };
 
     window.getFormData = function() {
@@ -397,7 +481,7 @@
             }
             var value = field.value;
             if ($input.attr('type') == 'checkbox') {
-                value = $input.prop('checked');
+                value = $input[0].checked;
             }
             if (fieldName == 'size' || fieldName == 'sizes' || fieldName == 'thumbs') {
                 try {
@@ -680,4 +764,4 @@
         });
     });
 
-}(django.jQuery));
\ No newline at end of file
+}(django.jQuery));
